#include <cmath>

#include "II_Buckingham.h"

#include "orient.h"

namespace {

double buck(const double   a, const double   b,
	    const double* p1, const double* p2)
{
    const double r = tools::distance(p1, p2);
    return a*exp(-b*r);
}

double buck(const double   a, const double   b,
	    const double* p1, const double* p2, double* g1, double* g2)
{
    const double r = tools::distance(p1, p2);

    const double fac = a*exp(-b*r);
    //const double fac = a/b*exp(-b*r);

    for(size_t i = 0; i < 3; ++i){
	g1[i] -= (p1[i] - p2[i])*b/r*fac;
	g2[i] += (p1[i] - p2[i])*b/r*fac;
    }

    return fac;
}

}

namespace x2o {

double A = 0;
double B = 0;

//Set parameters
void  II_Buckingham::set_params(const double a, const double b)
{

    A = a;
    B = b;

}

// calculate distances 
double II_Buckingham::operator()(const double* ion1, const double* ion2)
{

    const double BCA = buck(A, B, ion1, ion2);
    
    return BCA ;

}

double II_Buckingham::operator()(const double* ion1, const double* ion2,
				       double* g_ion1,     double* g_ion2)
{

    const double BCA = buck(A, B, ion1, ion2, g_ion1 , g_ion2);
    
    return BCA ;

}

// calculate repulsive energies
double II_Buckingham::buckingham(const double rII, const double a, const double b)
{

    double Vbuckingham =  a*exp(-b*rII);

    return Vbuckingham;
}

} // namespace x2o
