#include "II_Poly.h"
#include <iostream>


// Morse Variable definition
double morse(double k, double x) {
  return std::exp(-k * x)/x;
}

double switching(double x) {
  if (x < 0) {
    return 1;
  } else if (x < 1) {
    return 1 + x*x*(2*x-3);
  } else {
    return 0;
  }
}

double d_switching(double r, double ri, double ro, double axis) {
  double x = (r - ri)/(ro - ri);
  if (x < 0) {
    return 0;
  } else if (x < 1) {
    return 6 * axis * (r - ri) * (r + ro - 2 * ri) / pow(ro -ri,3) / r;
  } else {
    return 0;
  }
}

namespace x2o {

//Set parameters
double II_Poly::get_repulsion(double * k, double * c, int deg, double Ri, double Ro, double * pos, double * grd) {

  const double rCA = tools::distance(pos + 0, pos + 3);

  double x = (rCA - Ri)/(Ro - Ri);
  
  double rep_poly(0);

  // energy
  for (int p = 0; p < deg; p++){
    rep_poly += switching(x)*c[p]*pow(morse(k[p], rCA),p+1);
  }

  //for (int i = 0; i < 6; i++) std::cerr << "grd[" << i << "] = " << grd[i] << std::endl;
  // gradients
  for (int i = 0; i < 3; i++){ 
    for (int p = 0; p < deg; p++){
      double axis = pos[i] - pos[i+3];
      double r2 = rCA * rCA;
      double p2 = (double)p + 1.0;

      double g1 = d_switching(rCA, Ri, Ro, axis)*c[p]*pow(morse(k[p], rCA),p2) 
             + switching(x) * (-c[p] * k[p] * p2 * axis * pow(r2,(-p2/2-0.5)) * std::exp(-k[p]*p2*rCA)
             - c[p] * p2 * axis * pow(r2,(-p2/2-1)) * std::exp(-k[p]*p2*rCA));
      //double pw1 = std::pow(r2,(-p2/2-0.5));
      //double pw2 = pow(r2,(-p2/2-1));
      grd[i] += g1;
 
      double g2 = d_switching(rCA, Ri, Ro, axis)*c[p]*pow(morse(k[p], rCA),p+1)
                 + switching(x) * (-c[p] * k[p] * p2 * axis * pow(r2,(-p2/2-1.0/2.0)) * std::exp(-k[p]*p2*rCA)
                 -c[p] * p2 * axis * pow(r2,(-p2/2-1)) * std::exp(-k[p]*p2*rCA));
      grd[i + 3] -= g2;
    }
  }  

//  for (int i = 0; i < 6; i++) std::cerr << "grd[" << i << "] = " << grd[i] << std::endl;

  return rep_poly;

}

} // namespace x2o
