#ifndef ELECTROSTATICS_H
#define ELECTROSTATICS_H

#include <set>
#include <utility>
#include <cstddef>

#include "coulomb.h"

#include "system.h"

namespace ttm {

struct electrostatics {

    electrostatics();
    ~electrostatics();

    // wrapper for expanded electrostatics function
    double operator()(const sys::system&, const smear&, double* grad = 0);

    // energy
    double operator()(size_t N,
                      const double* q,      // size=N, charge
                      const double* polfac, // size=N, damping factor
                      const double* polar,  // size=N, polarizability
                      const double* xyz,    // size=3N, coordinates
                      const excluded_set_type& s12,
                      const excluded_set_type& s13,
                      const std::vector<int>&,
                      const std::vector<std::shared_ptr<molecule>>&,
                      const smear&,
                      double* grad = 0);

    // Overloaded energy due to the inclusion of the ecluded 14
    double operator()(size_t N,
                      const double* q,      // size=N, charge
                      const double* polfac, // size=N, damping factor
                      const double* polar,  // size=N, polarizability
                      const double* xyz,    // size=3N, coordinates
                      const excluded_set_type& s12,
                      const excluded_set_type& s13,
                      const excluded_set_type& s14,
                      const std::vector<int>&,
                      const std::vector<std::shared_ptr<molecule>>&,
                      const smear&,
                      double* grad = 0);

    // size=3*N, induced dipoles
    const double* dipoles() const;

    const double& E_ind() const { return m_Eind; }
    const double& E_elec() const { return m_Eelec; }

    void polarizability_tensor(size_t, double*); // 3x3, row major

private:
    size_t  m_size;
    double* m_memory;

    void allocate(size_t);

private:
    double m_Eind, m_Eelec;

};

} // namespace ttm

#endif // ELECTROSTATICS_H
