#include "ittm-Nion-water.h"
//#define VERBOSE
#define INTERACTION_ENERGY

namespace {
  int percent = 100;
  // Indicate for which ions you are going to use mb-nrg for the 2b
  std::vector<std::string> use_2b_water_ion; 
  // Indicate for which ions you are going to use mb-nrg for the 2b ion-ion
  std::vector<std::pair<std::string, std::string>> use_2b_ion_ion; 
  // Indicate for which ions you are going to use mb-nrg for the 3b
  std::vector<std::string> use_3b_water_ion;  
} // end namespace

void nion::set_2b_water_ions() {
}
void nion::set_3b_water_ions() {
}

void nion::set_2b_ion_ion() {
}



bool nion::orderIons (tools::xyz_frame i,tools::xyz_frame j) {
  return (i.at_name[0] < j.at_name[0]);
}

double nion::get_E(double * coords, std::vector<std::string> at_names, int n_at)
{
  
  std::vector<tools::xyz_frame> water;
  std::vector<tools::xyz_frame> ions;
  // Declaring number of water molecules and ions
  int nw(0),ni(0);
  // Looping over all the atoms of the frame i
  std::vector<std::string> namesi;
  for (int j=0; j< n_at; j++) {
    if (at_names[j] == "O") {
      // If is water, add the three atoms into water vector
      std::vector<std::string> namesw;
      namesw.push_back("O");
      namesw.push_back("H");
      namesw.push_back("H");

      // Creating a frame with the water molecule
      tools::xyz_frame wat(coords + 3*j,namesw,3);

      // Pushing the frame into the vector of waters
      water.push_back(wat);
      nw++;
      j+=2;
    } else {
      // If not, is an ion. Add the ion in the ion vector
      std::vector<std::string> names_ii;
      names_ii.push_back(at_names[j]);
      namesi.push_back(at_names[j]);

      // Create a frame with the ion
      tools::xyz_frame ion(coords + 3*j,names_ii,1);

      // Push the ion into the ions vector
      ions.push_back(ion);
      ni++;
    }
  }

  // At this point we have everything separated
  // Ordering the ions vector with the function orderIons
  std::sort (ions.begin(), ions.end(), orderIons);
  
  // Creating a pointer with the positions of water
  double *water_coords;
  water_coords = new double[9*nw];
  for (int j = 0; j < nw; j++) {
    std::copy(water[j].xyz ,water[j].xyz + 9, water_coords + 9*j);
  }

  // 1. Water energy
  double E_wat(0);
  double E1(0); 
  {
    if (nw > 0) {
      // Create a pot_water_ion object and get the water energy
      x2o::pot_water_ion pot;
      E_wat += pot.get_E_water(nw, water_coords);
      E1 += pot.get_E_water_1b(nw, water_coords);
    }
  } 
  delete[] water_coords;

# ifdef VERBOSE
  std::cerr << "E water: " << E_wat << std::endl;
# endif

  // 2. Water - ion energy
  double E_wations(0);
  if (nw > 0) {
    for (int j=0; j < ni; j++) {
      // For each ion, we create a system with all the water molecules
      // one ion at a time. This way we can recicle the functions
      // from pot-water-ion.cpp
      double *system_coords;
      system_coords = new double[9*nw + 3];
      
      // Copy the water molecules
      for (int k = 0; k < nw; k++) {
        std::copy(water[k].xyz ,water[k].xyz + 9, system_coords + 9*k);
      }
      
      // Copy the ion into the system
      std::copy(ions[j].xyz,ions[j].xyz + 3, system_coords + 9*nw);
      
      // Create again a pot object and get the ion-water energy for the
      // jth ion
      x2o::pot_water_ion pot;

      // Using the class multiion_param to recover the ion-water parameters
      multiion_param a;
      if (percent == 50) {
        a.set_parameters50(ions[j].at_name[0]);
      } else if (percent == 0) {
        a.set_parameters00(ions[j].at_name[0]);
      } else {
        a.set_parameters(ions[j].at_name[0]);
      }
      pot.define_ion_params(a.params);
   
      // Updating the energy 
      E_wations += pot.get_E_water_ion(nw,1, system_coords);
      delete[] system_coords;
    }
  }

# ifdef VERBOSE
  std::cerr << "E water-ion: " << E_wations << std::endl;
# endif

  // 3. Ion - Ion
  double E_ions(0);
  for (int j=0; j < ni - 1; j++) {
    for (int k= j + 1; k< ni; k++) {
      // Defining the system composed by two ions
      double *system_coords;
      system_coords = new double[6];
      std::copy(ions[j].xyz,ions[j].xyz + 3, system_coords);
      std::copy(ions[k].xyz,ions[k].xyz + 3, system_coords + 3);
      
      // Recovering the parameters and updating the energy
      x2o::pot_water_ion pot;
      multiion_param a;
      if (percent == 50) {
        a.set_parameters50(ions[j].at_name[0],ions[k].at_name[0]);
      } else if (percent == 0) {
        a.set_parameters00(ions[j].at_name[0],ions[k].at_name[0]);
      } else {
        a.set_parameters(ions[j].at_name[0],ions[k].at_name[0]);
      }
      pot.define_ion_params(a.params);
      E_ions += pot.get_E_ion_ion(1,1,system_coords);
      delete[] system_coords;
    }
  }

# ifdef VERBOSE
  std::cerr << "E ion-ion: " << E_ions << std::endl;
# endif

  // 4. Now electrostatics

  // Defining the system to calculate the electrostatic energy
  sys::system my_system;

  // Adding water molecules to the system
  for (int j=0;j<nw;j++) {
    std::shared_ptr<molecule> tmp = std::make_shared<x2o::ttm4>(water[j].xyz);
    tmp->id = "n";
    my_system.add_molecule(tmp);
  }

  // Adding the ions to the system
  for (int j=0;j<ni;j++) {
    molecule this_ion;
    this_ion.allocate(1);
    this_ion.set_coords(ions[j].xyz);
    
    multiion_param a;
    if (percent == 50) {
      a.set_parameters50(ions[j].at_name[0]);
      double chg = a.params[0];
      double pol = a.params[1];
      this_ion.set_charges(&chg);
      this_ion.set_pol(&pol); // polfac=pol ==> No problem
    } else if (percent == 0) {
      a.set_parameters00(ions[j].at_name[0]);
      double chg = a.params[0];
      double pol = 0.0;
      double polfac = 0.000001;
      this_ion.set_charges(&chg);
      this_ion.set_pol(&pol);
      this_ion.set_polfac(&polfac);
    } else {
      a.set_parameters(ions[j].at_name[0]);
      double chg = a.params[0];
      double pol = a.params[1];
      this_ion.set_charges(&chg);
      this_ion.set_pol(&pol); // polfac=pol ==> No problem
    }
    this_ion.id = "i";

    std::shared_ptr<molecule> tmp = std::make_shared<molecule>(this_ion);
    my_system.add_molecule(tmp);
  }
  
  // We calculate the electrostatics energy
  ttm::electrostatics elect;
  double *grad = 0;
  ttm::smear_ttm4x smr;

  // Defining the smearing
  smr.m_aDD_intra_12 = 0.626;
  smr.m_aDD_intra_13 = 0.055;
  double energy = elect(my_system.nsites, my_system.charges,
                          my_system.polfac, my_system.atmpolar,
                          my_system.coords,
                          my_system.excluded12, my_system.excluded13,
                          my_system.first_index_of_molec,
                          my_system.molec, smr, grad);

# ifdef VERBOSE
  std::cerr << "E electrostatic: " << energy << std::endl;   
  std::cerr << "Binding_Energy= " << E_wat + E_wations + E_ions + energy << std::endl;
# endif
  
# ifdef INTERACTION_ENERGY
  std::cout << "Interaction_energy= " << E_wat + E_wations + E_ions + energy - E1 
            << " Binding_Energy= " << E_wat + E_wations + E_ions + energy << std::endl;
# endif 
  return E_wat + E_wations + E_ions + energy;
  
}


double nion::get_E(double * coords, std::vector<std::string> at_names, int n_at, double * my_grd) {

    //////////////////////////////////////////////////////// ////// //////
   // If questions, look the function without gradients. // //  // //  //
  //////////////////////////////////////////////////////// ////// //////

  double * grd;
  grd = new double[3*n_at];
  std::vector<tools::xyz_frame> water;
  std::vector<tools::xyz_frame> ions;
  std::vector<int> water_indeces;
  std::vector<int> ions_indeces;
  // Declaring number of water molecules and ions
  int nw(0),ni(0);
  // Looping over all the atoms of the frame i
  std::vector<std::string> namesi;
  for (int j=0; j< n_at; j++) {
    if (at_names[j] == "O") {
      // If is water, add the three atoms into water vector
      std::vector<std::string> namesw;
      namesw.push_back("O");
      namesw.push_back("H");
      namesw.push_back("H");
      
      // Saving the indeces
      water_indeces.push_back(j);
      water_indeces.push_back(j+1);
      water_indeces.push_back(j+2);

      tools::xyz_frame wat(coords + 3*j,namesw,3);

      water.push_back(wat);
      nw++;
      j+=2;
    } else {
      // If not, is an ion. Add the ion in the ion vector
      std::vector<std::string> names_ii;
      names_ii.push_back(at_names[j]);
      namesi.push_back(at_names[j]);

      // Saving indeces
      ions_indeces.push_back(j);

      tools::xyz_frame ion(coords + 3*j,names_ii,1);

      ions.push_back(ion);
      ni++;
    }
  }

  // At this point we have everything separated
  // Ordering the ions vector with the function orderIons
  //std::sort (ions.begin(), ions.end(), orderIons);

  // Creating a pointer with the positions of water
  double *water_coords;
  water_coords = new double[9*nw];
  for (int j = 0; j < nw; j++) {
    std::copy(water[j].xyz ,water[j].xyz + 9, water_coords + 9*j);
  }

  std::fill(grd, grd + 9*nw + 3*ni, 0.0);
  // 1. Water energy
  double E_wat(0);
  double E1(0);
  {
    double grad[9*nw];
    if (nw > 0) {
      x2o::pot_water_ion pot;
      E_wat += pot.get_E_water(nw, water_coords,grad);
      E1 += pot.get_E_water_1b(nw, water_coords);
    }
  
  
    for (int j = 0; j< 9*nw; j++) {
      grd[j] += grad[j];
    }
  }
  delete[] water_coords;

# ifdef VERBOSE
    std::cerr << "E water: " << E_wat << std::endl;
# endif
  
  
  // 2. Water - ion energy

  // 2.1 2b iTTM or mb-nrg

  // Setting the ions for which we are going to use mb_nrg 2b and 3b
  set_2b_water_ions();
  set_3b_water_ions();

#ifdef VERBOSE
  double E2bwation(0);
  double E2bwation_disp(0);
#endif


  // Getting the energy
  double E_wations(0);
  if (nw > 0) {
    for (int j=0; j < ni; j++) {
      double grd2[9*nw + 3];
      std::fill(grd2, grd2 + 9*nw + 3, 0.0);
      double *system_coords;
      system_coords = new double[9*nw + 3];
      for (int k = 0; k < nw; k++) {
        std::copy(water[k].xyz ,water[k].xyz + 9, system_coords + 9*k);
      }
      std::copy(ions[j].xyz,ions[j].xyz + 3, system_coords + 9*nw);

//      multiion_param b;
//      bool have_2b = b.check_ion_water_2b_mbnrg(names(j));
//      bool have_3b = b.check_ion_water_3b_mbnrg(names(j));
      if (std::find(use_2b_water_ion.begin(), use_2b_water_ion.end(), 
              namesi[j]) != use_2b_water_ion.end() ) {
        mbnrg_param_X_H2O * a = new mbnrg_param_X_H2O ;
        a->set_parameters(namesi[j]);
        
        // Loop overall the water molecules and get the energy
        h2o_ion::x2b_h2o_ion_v2x  this_is_a_mess;
        this_is_a_mess.set_param(a);
        
        for (int k = 0; k < nw; k++) {
          double grd_wa[9],grd_io[3];
          std::fill(grd_wa, grd_wa + 9, 0.0);
          std::fill(grd_io, grd_io + 3, 0.0);
          
          double E2b_w_i = this_is_a_mess.eval(water[k].xyz, 
                             ions[j].xyz, grd_wa, grd_io);

          double E2b_disp = x2o::mbnrg_dispersion(water[k].xyz, ions[j].xyz, 
                                   a->C6XO, a->C6XH, a->D6XO, a->D6XH, grd_wa, grd_io);

#ifdef VERBOSE
          std::cout << "E_disp wat[" << k << "] - ion = " << E2b_disp << std::endl;
#endif

          // This is to prepare for parallelization.
          for (int k2 = 0; k2 < 9; k2++) {
            grd2[9*k + k2] += grd_wa[k2];
          }

          for (int k2 = 0; k2 < 3; k2++) {
            grd2[9*nw + k2] += grd_io[k2];
          }
          E_wations += (E2b_w_i + E2b_disp);
#ifdef VERBOSE
          E2bwation += E2b_w_i;
          E2bwation_disp += E2b_disp;
#endif
        }
        delete a;
        
      } else {
        x2o::pot_water_ion pot;
        multiion_param a;
        if (percent == 50) {
          a.set_parameters50(ions[j].at_name[0]);
        } else if (percent == 0) {
          a.set_parameters00(ions[j].at_name[0]);
        } else {
          a.set_parameters(ions[j].at_name[0]);
        }
        pot.define_ion_params(a.params);
        E_wations += pot.get_E_water_ion(nw,1, system_coords, grd2);
      }

      for (int k = 0; k< 9*nw; k++) {
        grd[k] += grd2[k];
      }
      for (int k = 0; k < 3; k++) {
        grd[9*nw + j*3 + k] += grd2[9*nw + k];
      }
      delete[] system_coords;
    }
  }

#ifdef VERBOSE
  double E3bwation(0);
#endif

  // 2.2 3b mb-nrg
  if (nw > 0) {
    for (int j=0; j < ni; j++) {
      double grd2[9*nw + 3];
      std::fill(grd2, grd2 + 9*nw + 3, 0.0);
      if (std::find(use_3b_water_ion.begin(), use_3b_water_ion.end(),
              namesi[j]) != use_3b_water_ion.end() ) {

        mbnrg_param_X_H2O_H2O * a = new mbnrg_param_X_H2O_H2O ;
        a->set_parameters(namesi[j]);

        // Loop overall the water molecules and get the energy
        h2o_ion::x3b_h2o_ion_v2x  this_is_a_mess;
        this_is_a_mess.set_param(a);

        for (int k = 0; k < nw - 1; k++) {
          for (int l = k + 1; l < nw; l++) {
            double grd_wa1[9],grd_wa2[9],grd_io[3];
            std::fill(grd_wa1, grd_wa1 + 9, 0.0);
            std::fill(grd_wa2, grd_wa2 + 9, 0.0);
            std::fill(grd_io, grd_io + 3, 0.0);
            double E3b_w_w_i = this_is_a_mess.eval(water[k].xyz, water[l].xyz,
                             ions[j].xyz, grd_wa1, grd_wa2, grd_io);
            // This is to prepare for parallelization.
            for (int k2 = 0; k2 < 9; k2++) {
              grd2[9*k + k2] += grd_wa1[k2];
            }
            for (int k2 = 0; k2 < 9; k2++) {
              grd2[9*l + k2] += grd_wa2[k2];
            }

            for (int k2 = 0; k2 < 3; k2++) {
              grd2[9*nw + k2] += grd_io[k2];
            }
            E_wations += E3b_w_w_i;

#ifdef VERBOSE
            E3bwation += E3b_w_w_i;
#endif             
          }
        }
        delete a;
        for (int k = 0; k< 9*nw; k++) {
          grd[k] += grd2[k];
        }
        for (int k = 0; k < 3; k++) {
          grd[9*nw + j*3 + k] += grd2[9*nw + k];
        }

      }
    }
  }


# ifdef VERBOSE
  std::cerr << "E 2body water ion =  " << E2bwation << std::endl;
  std::cerr << "E 2body water ion dispersion =  " << E2bwation_disp << std::endl;
  std::cerr << "E 3body water ion =  " << E3bwation << std::endl;
  std::cerr << "E water-ion: " << E_wations << std::endl;
# endif

  // 3. Ion - Ion

  // Setting the ions for which we hve mb-nrg
  set_2b_ion_ion();

  // Going for it
  double E_ions(0);
  for (int j=0; j < ni - 1; j++) {
    for (int k= j + 1; k< ni; k++) {
      double grd2[6];
      std::fill(grd2,grd2 + 6,0.0);
      double *system_coords;
      system_coords = new double[6];
      std::copy(ions[j].xyz,ions[j].xyz + 3, system_coords);
      std::copy(ions[k].xyz,ions[k].xyz + 3, system_coords + 3);

      x2o::pot_water_ion pot;
      multiion_param a;
      if (percent == 50) {
        a.set_parameters50(ions[j].at_name[0],ions[k].at_name[0]);
      } else if (percent == 0) {
        a.set_parameters00(ions[j].at_name[0],ions[k].at_name[0]);
      } else {
        a.set_parameters(ions[j].at_name[0],ions[k].at_name[0]);
      }

      std::pair<std::string,std::string> ipair =
                std::make_pair(ions[j].at_name[0], ions[k].at_name[0]);
      
      if (std::find(use_2b_ion_ion.begin(), use_2b_ion_ion.end(), ipair) != use_2b_ion_ion.end()){
        // Use mb-nrg
        mbnrg_param_ion_ion aa;
        aa.set_parameters(ions[j].at_name[0], ions[k].at_name[0]);

        x2o::II_Dispersion m_II_dispersion;
        m_II_dispersion.set_params(aa.C6CA, aa.deltaCA);
        double Edis = m_II_dispersion(system_coords, system_coords + 3, grd2, grd2 + 3); 

        x2o::II_Poly m_II_Poly;
        double Epoly = m_II_Poly.get_repulsion(aa.ki, aa.ci, aa.deg, aa.Ri, aa.Ro, system_coords, grd2);

        E_ions += Epoly + Edis;
      } else {
        // Use i-TTM 
        pot.define_ion_params(a.params);
        E_ions += pot.get_E_ion_ion(1,1,system_coords, grd2);
      }

      for (int l = 0; l< 3; l++) {
        grd[nw*9 + j*3 + l ] += grd2[l];
      }
      for (int l = 0; l < 3; l++) {
        grd[9*nw + k*3 + l ] += grd2[l + 3];
      }
      delete[] system_coords;
    }
  }

# ifdef VERBOSE
    std::cerr << "E ion-ion: " << E_ions << std::endl;
# endif
  
  // 4. Now electrostatics
  sys::system my_system;
  for (int j=0;j<nw;j++) {
    std::shared_ptr<molecule> tmp = std::make_shared<x2o::ttm4>(water[j].xyz);
    tmp->id = "n";
    my_system.add_molecule(tmp);
  }
  for (int j=0;j<ni;j++) {
    molecule this_ion;
    this_ion.allocate(1);
    this_ion.set_coords(ions[j].xyz);

    multiion_param a;
    if (percent == 50) {
      a.set_parameters50(ions[j].at_name[0]);
      double chg = a.params[0];
      double pol = a.params[1];
      this_ion.set_charges(&chg);
      this_ion.set_pol(&pol); // polfac=pol ==> No problem
    } else if (percent == 0) {
      a.set_parameters00(ions[j].at_name[0]);
      double chg = a.params[0];
      double pol = 0.0;
      double polfac = 0.000001;
      this_ion.set_charges(&chg);
      this_ion.set_pol(&pol);
      this_ion.set_polfac(&polfac);
    } else {
      a.set_parameters(ions[j].at_name[0]);
      double chg = a.params[0];
      double pol = a.params[1];
      this_ion.set_charges(&chg);
      this_ion.set_pol(&pol); // polfac=pol ==> No problem
    }
    this_ion.id = "i";

    std::shared_ptr<molecule> tmp = std::make_shared<molecule>(this_ion);
    my_system.add_molecule(tmp);
  }


  // We calculate the electrostatics energy
  ttm::electrostatics elect;
  double grd2[3*my_system.nsites];
  ttm::smear_ttm4x smr;
  smr.m_aDD_intra_12 = 0.626;
  smr.m_aDD_intra_13 = 0.055;
  double energy = elect(my_system.nsites, my_system.charges,
                          my_system.polfac, my_system.atmpolar,
                          my_system.coords,
                          my_system.excluded12, my_system.excluded13,
                          my_system.first_index_of_molec,
                          my_system.molec, smr, grd2);

  for (int k = 0; k < nw; k++) {
    int n3 = k*3;
    int n4 = k*4;
    for (int l=0; l < 9; l++) {
      grd[3*n3 + l] += grd2[3*n4 +l];
    } 
  }
  for (int k= 0; k< ni; k++) {
    for (int l = 0; l < 3; l++) {
      grd[9*nw + 3*k + l] += grd2[12*nw + 3*k +l];
    }
  }

  

  // Putting the gradients in the original coordinates order
  for (int i =0; i< nw; i++) {
    for (int j=0;j<9;j++) {
      int wat_pos = water_indeces[3*i];
      my_grd[wat_pos*3 + j] = grd[i*9 + j];
    }
  }
  for (int i =0; i< ni; i++) {
    for (int j=0;j<3;j++) {
      int ion_pos = ions_indeces[i];
      my_grd[ion_pos*3 + j] = grd[9*nw+ i*3 + j];
    }
  }



# ifdef VERBOSE
  std::cerr << "E electrostatic: " << energy << std::endl;
  std::cerr << "Binding_Energy= " << E_wat + E_wations + E_ions + energy << std::endl;

  std::cerr << "{{{ GRADIENTS }}}" << std::endl;
  for (int i = 0; i < (9*nw + 3 * ni); i++) {
    std::cerr << "grd[" << i << "]=\t" << grd[i] << std::endl;
  }
# endif
  delete[] grd;

# ifdef INTERACTION_ENERGY
  std::cout << "Interaction_energy_grad= " << E_wat + E_wations + E_ions + energy - E1
            << " Binding_Energy_grad= " << E_wat + E_wations + E_ions + energy << std::endl;
# endif

  return E_wat + E_wations + E_ions + energy;
}

void nion::calcdip(double * coords, std::vector<std::string> at_names, int n_at, double * dipoles, double * dip)
{

  // NOTE: Dipoles needs to have dimension 3*nw + 3*ni
  //       Dip is a 3 dimension vector and contains the total dipole moment
  std::vector<tools::xyz_frame> water;
  std::vector<tools::xyz_frame> ions;
  std::vector<int> water_indeces;
  std::vector<int> ions_indeces;
  // Declaring number of water molecules and ions
  int nw(0),ni(0);
  // Looping over all the atoms of the frame i
  std::vector<std::string> namesi;
  for (int j=0; j< n_at; j++) {
    if (at_names[j] == "O") {
      // If is water, add the three atoms into water vector
      std::vector<std::string> namesw;
      namesw.push_back("O");
      namesw.push_back("H");
      namesw.push_back("H");

      // Saving the indeces
      water_indeces.push_back(j);
      water_indeces.push_back(j+1);
      water_indeces.push_back(j+2);

      tools::xyz_frame wat(coords + 3*j,namesw,3);

      water.push_back(wat);
      nw++;
      j+=2;
    } else {
      // If not, is an ion. Add the ion in the ion vector
      std::vector<std::string> names_ii;
      names_ii.push_back(at_names[j]);
      namesi.push_back(at_names[j]);

      // Saving indeces
      ions_indeces.push_back(j);

      tools::xyz_frame ion(coords + 3*j,names_ii,1);

      ions.push_back(ion);
      ni++;
    }
  }

  // At this point we have everything separated

  // Creating a pointer with the positions of water
  double *water_coords;
  water_coords = new double[9*nw];
  for (int j = 0; j < nw; j++) {
    std::copy(water[j].xyz ,water[j].xyz + 9, water_coords + 9*j);
  }

  // Defining the system to calculate the electrostatic energy
  sys::system my_system;

  // Adding water molecules to the system
  for (int j=0;j<nw;j++) {
    std::shared_ptr<molecule> tmp = std::make_shared<x2o::ttm4>(water[j].xyz);
    tmp->id = "n";
    my_system.add_molecule(tmp);
  }

  // Adding the ions to the system
  for (int j=0;j<ni;j++) {
    molecule this_ion;
    this_ion.allocate(1);
    this_ion.set_coords(ions[j].xyz);

    multiion_param a;
    if (percent == 50) {
      a.set_parameters50(ions[j].at_name[0]);
      double chg = a.params[0];
      double pol = a.params[1];
      this_ion.set_charges(&chg);
      this_ion.set_pol(&pol); // polfac=pol ==> No problem
    } else if (percent == 0) {
      a.set_parameters00(ions[j].at_name[0]);
      double chg = a.params[0];
      double pol = 0.0;
      double polfac = 0.000001;
      this_ion.set_charges(&chg);
      this_ion.set_pol(&pol);
      this_ion.set_polfac(&polfac);
    } else {
      a.set_parameters(ions[j].at_name[0]);
      double chg = a.params[0];
      double pol = a.params[1];
      this_ion.set_charges(&chg);
      this_ion.set_pol(&pol); // polfac=pol ==> No problem
    }
    this_ion.id = "i";

    std::shared_ptr<molecule> tmp = std::make_shared<molecule>(this_ion);
    my_system.add_molecule(tmp);
  }

  // We calculate the electrostatics energy
  ttm::electrostatics elect;
  ttm::smear_ttm4x smr;  

  // Defining the smearing
  smr.m_aDD_intra_12 = 0.626;
  smr.m_aDD_intra_13 = 0.055;
  double energy = elect(my_system.nsites, my_system.charges,
                          my_system.polfac, my_system.atmpolar,
                          my_system.coords,
                          my_system.excluded12, my_system.excluded13,
                          my_system.first_index_of_molec,
                          my_system.molec, smr,0);
  
  energy = energy + 1.0; // To avoid warning

  std::fill(dipoles, dipoles + 3*nw + 3*ni, 0.0);
  std::fill(dip, dip + 3, 0.0);

  const double * dipind = elect.dipoles();

  // Adding permanent dipole
  for (size_t n = 0; n < nw; ++n) {

    const size_t n3 = 3*n;
    const size_t n4 = 4*n;
    const size_t n12 = 3*n4;

    for (size_t i = 0; i < 4; ++i)
      for (size_t k = 0; k < 3; ++k) {
        double contr = 
          my_system.charges[n4 + i]*my_system.coords[n12 + 3*i + k];
        dipoles[3*n + k] += contr;
        dip[k] += contr;
      }
  }

//  for (size_t k = 0; k < 3; ++k)
//    dip[k] *= constants::DEBYE/constants::CHARGECON;
  

  // Adding induced
  for (int i =0; i< nw; i++) {
    double dips[3];
    std::fill(dips, dips + 3, 0.0);
    for (int j=0;j<3;j++) {
      for (int k=0;k<3;k++) {
        dips[k] += dipind[i*12 + 3*j + k];
      }
    }
    for (int j=0;j<3;j++) {
      dipoles[3*i + j] += dips[j];
      dip[j] += dips[j];
    }
  }

  for (int i =0; i< ni; i++) {
    for (int j=0;j<3;j++) {
      dipoles[3*nw + 3*i + j] += dipind[12*nw+ i*3 + j];
      dip[j] += dipind[12*nw+ i*3 + j];
    }
  }

  for (size_t k = 0; k < 3; ++k) {
    dip[k] *= constants::DEBYE/constants::CHARGECON;
    for (size_t j = 0; j < nw + ni ; j++) {
      dipoles[3*j + k]*= constants::DEBYE/constants::CHARGECON;
    }
  }

  double dcp[3*nw + 3*ni];
  int wc = 0;
  int ic = 0;
  for (size_t i = 0; i < 3*nw + ni; i++) {
    if (at_names[i] == "O") {
      for (size_t j = 0; j < 3; j++)
        dcp[(wc + ic)*3 + j] = dipoles[wc + j];
      i += 2;
      wc++;
    } else {
      for (size_t j = 0; j < 3; j++) 
        dcp[(wc + ic)*3 + j] = dipoles[nw + ic + j];
      ic++;
    }
  }

  for (size_t i = 0; i < nw + ni ; i++)
    dipoles[i] = dcp[i];
    

  delete[] water_coords;
  
}


void nion::calcdip(double * coords, std::vector<std::string> at_names, 
        int n_at, double * dip) {

  // NOTE: Dipoles needs to have dimension 3*nw + 3*ni
  //       Dip is a 3 dimension vector and contains the total dipole moment
  std::vector<tools::xyz_frame> water;
  std::vector<tools::xyz_frame> ions;
  std::vector<int> water_indeces;
  std::vector<int> ions_indeces;
  // Declaring number of water molecules and ions
  int nw(0),ni(0);
  // Looping over all the atoms of the frame i
  std::vector<std::string> namesi;
  for (int j=0; j< n_at; j++) {
    if (at_names[j] == "O") {
      // If is water, add the three atoms into water vector
      std::vector<std::string> namesw;
      namesw.push_back("O");
      namesw.push_back("H");
      namesw.push_back("H");

      // Saving the indeces
      water_indeces.push_back(j);
      water_indeces.push_back(j+1);
      water_indeces.push_back(j+2);

      tools::xyz_frame wat(coords + 3*j,namesw,3);

      water.push_back(wat);
      nw++;
      j+=2;
    } else {
      // If not, is an ion. Add the ion in the ion vector
      std::vector<std::string> names_ii;
      names_ii.push_back(at_names[j]);
      namesi.push_back(at_names[j]);

      // Saving indeces
      ions_indeces.push_back(j);

      tools::xyz_frame ion(coords + 3*j,names_ii,1);

      ions.push_back(ion);
      ni++;
    }
  }

  // At this point we have everything separated

  // Creating a pointer with the positions of water
  double *water_coords;
  water_coords = new double[9*nw];
  for (int j = 0; j < nw; j++) {
    std::copy(water[j].xyz ,water[j].xyz + 9, water_coords + 9*j);
  }

  // Defining the system to calculate the electrostatic energy
  sys::system my_system;

  // Adding water molecules to the system
  for (int j=0;j<nw;j++) {
    std::shared_ptr<molecule> tmp = std::make_shared<x2o::ttm4>(water[j].xyz);
    tmp->id = "n";
    my_system.add_molecule(tmp);
  }

  // Adding the ions to the system
  for (int j=0;j<ni;j++) {
    molecule this_ion;
    this_ion.allocate(1);
    this_ion.set_coords(ions[j].xyz);

    multiion_param a;
    if (percent == 50) {
      a.set_parameters50(ions[j].at_name[0]);
      double chg = a.params[0];
      double pol = a.params[1];
      this_ion.set_charges(&chg);
      this_ion.set_pol(&pol); // polfac=pol ==> No problem
    } else if (percent == 0) {
      a.set_parameters00(ions[j].at_name[0]);
      double chg = a.params[0];
      double pol = 0.0;
      double polfac = 0.000001;
      this_ion.set_charges(&chg);
      this_ion.set_pol(&pol);
      this_ion.set_polfac(&polfac);
    } else {
      a.set_parameters(ions[j].at_name[0]);
      double chg = a.params[0];
      double pol = a.params[1];
      this_ion.set_charges(&chg);
      this_ion.set_pol(&pol); // polfac=pol ==> No problem
    }
    this_ion.id = "i";

    std::shared_ptr<molecule> tmp = std::make_shared<molecule>(this_ion);
    my_system.add_molecule(tmp);
  }

  // We calculate the electrostatics energy
  ttm::electrostatics elect;
  ttm::smear_ttm4x smr;  

  // Defining the smearing
  smr.m_aDD_intra_12 = 0.626;
  smr.m_aDD_intra_13 = 0.055;
  double energy = elect(my_system.nsites, my_system.charges,
                          my_system.polfac, my_system.atmpolar,
                          my_system.coords,
                          my_system.excluded12, my_system.excluded13,
                          my_system.first_index_of_molec,
                          my_system.molec, smr,0);
  
  energy = energy + 1.0; // To avoid warning

  std::fill(dip, dip + 3, 0.0);

  const double * dipind = elect.dipoles();

  // Adding permanent dipole
  for (size_t n = 0; n < nw; ++n) {

    const size_t n3 = 3*n;
    const size_t n4 = 4*n;
    const size_t n12 = 3*n4;

    for (size_t i = 0; i < 4; ++i)
      for (size_t k = 0; k < 3; ++k) {
        double contr = 
          my_system.charges[n4 + i]*my_system.coords[n12 + 3*i + k];
        dip[k] += contr;
      }
  }

//  for (size_t k = 0; k < 3; ++k)
//    dip[k] *= constants::DEBYE/constants::CHARGECON;
  

  // TODO Check that the induced dipole for M site is 0
  // Adding induced
  for (int i =0; i< nw; i++) {
    double dips[3];
    std::fill(dips, dips + 3, 0.0);
    for (int j=0;j<3;j++) {
      for (int k=0;k<3;k++) {
        dips[k] += dipind[i*12 + 3*j + k];
      }
    }
    for (int j=0;j<3;j++) {
      dip[j] += dips[j];
    }
  }

  for (int i =0; i< ni; i++) {
    for (int j=0;j<3;j++) {
      dip[j] += dipind[12*nw+ i*3 + j];
    }
  }

  for (size_t k = 0; k < 3; ++k) {
    dip[k] *= constants::DEBYE/constants::CHARGECON;
  }

  delete[] water_coords;
  
}


void nion::calcdipgrad(double * coords, std::vector<std::string> at_names, int n_at, double * dipgrad){

  // dipgrad has 9*N components. Each atom is moved x, y and z, and 
  // for each movement we have a 3 component dipole grad
  std::fill(dipgrad, dipgrad + n_at*3*3, 0.0);

  const double eps = 1.0e-5;
  for (int i=0; i < 3*n_at; ++i) {
    const double x0 = coords[i];

    double dipp[3];
    double dipm[3];

    coords[i] = x0 + eps;
    nion::calcdip(coords, at_names, n_at, dipp);

    coords[i] = x0 - eps;
    nion::calcdip(coords, at_names, n_at, dipm);

    coords[i] = x0;
    
    for (size_t j = 0; j < 3; ++j) {
      dipgrad[i*3 + j] = 0.5 * (dipp[j] - dipm[j]) / eps;
    }
  }
}
