#ifndef MOLECULE_H
#define MOLECULE_H

//This contains the base structure "molecule". 
//The derived structures will be the specific molecules.

#include <iterator>
#include <set>
#include <string>

#include "constants.h"

typedef std::set<std::pair<size_t, size_t> > excluded_set_type;

////////////////////////////////////////////////////////////////////////////////

class molecule {

//------------------  Protected data abstracted within the base class  -------//

protected : 
    size_t nsites;
    double* coords;
    double* atmpolar;
    double* charge;
    double* polfac;

    double* memory;

    bool definedNsites, definedCoords, definedCharges, definedPolarizability;
    bool charge_derivatives, virtual_sites;

public:
    //specify the excluded pairs here
    excluded_set_type excluded12;
    excluded_set_type excluded13;
    excluded_set_type excluded14;

//------------------  Constructors / Destructors / Memory Allocation  --------//

public :
    molecule();
    virtual ~molecule();

    // copy constructor
    molecule(const molecule& o);

    // sets nsites and allocates memory
    virtual void allocate(const int n);

//------------------  Functions for setting data in base class ---------------//

public:

    std::string id;
    
    virtual void set_coords(double* xyz);
    virtual void set_charges(double* q);
    virtual void set_pol(double* p);
    virtual void set_polfac(double* p);

    virtual void redistrib_frc_virtual_2_real(double* grad){ }
    virtual void charge_derivative_force(double* phi, double* grad){ }

    excluded_set_type::iterator get_begin_12() { return excluded12.begin(); }
    excluded_set_type::iterator get_begin_13() { return excluded13.begin(); }
    excluded_set_type::iterator get_begin_14() { return excluded14.begin(); }
    excluded_set_type::iterator get_end_12() { return excluded12.end(); }
    excluded_set_type::iterator get_end_13() { return excluded13.end(); }
    excluded_set_type::iterator get_end_14() { return excluded14.end(); }

//------------------  Functions for accessing data in base class -------------//

    size_t get_nsites();

    double* get_charges();
    double* get_coords();
    double* get_pol();
    double* get_polfac();

    void get_charges(double* new_charges);
    void get_coords(double* new_coords);
    void get_pol(double* new_atmpolar);
    void get_polfac(double* new_polfac);

    bool has_virtual_sites();
    bool has_charge_derivatives();

//----------------------------------------------------------------------------//

};

#endif // MOLECULE_H
