#ifndef PARAMS_H
#define PARAMS_H

#include <string>
#include <vector>
#include <algorithm>
#include <utility>

////////////////////////////////////////////////////////////////////////////////

class multiion_param {


public : 

  multiion_param() {
    mass = -1.0;
    poly_deg = -1;
  };
  multiion_param(int pd) {
    mass = -1.0;
    poly_deg = pd;
  };
  ~multiion_param() {
  };


  double params[10];
  double mass;
  int poly_deg;
  
  void set_parameters(const std::string ion);
  void set_parameters(const std::string ion1, const std::string ion2);
  void set_parameters50(const std::string ion);
  void set_parameters50(const std::string ion1, const std::string ion2);
  void set_parameters00(const std::string ion);
  void set_parameters00(const std::string ion1, const std::string ion2);
 
  bool check_ion_water_2b_mbnrg(const std::string ion);
  bool check_ion_water_3b_mbnrg(const std::string ion);
  bool check_ion_ion_2b_mbnrg(const std::string ion1, const std::string ion2);

  double set_mass(const std::string atom);  

};

class mbnrg_param_X_H2O {

public:
  mbnrg_param_X_H2O() {
    twobodyfit = new double[429];
  };

  ~mbnrg_param_X_H2O() {
    delete[] twobodyfit;
  };

  void set_parameters(const std::string ion);

  double polarizability;
 
  double C6XH; // kcal/mol * A^(-6)
  double C6XO; // kcal/mol * A^(-6)
 
  double D6XH; // A^(-1)
  double D6XO; // A^(-1)
 
  double K_HH_INTRA; // A^(-1)
  double K_OH_INTRA; // A^(-1)
 
  double K_XH_COUL; // A^(-1)
  double K_XO_COUL; // A^(-1)

  double D_HH_INTRA; // A^(-1)
  double D_OH_INTRA; // A^(-1)

  double D_XH_COUL; // A^(-1)
  double D_XO_COUL; // A^(-1)
 
  double K_XLP_MAIN; // A^(-1)
  double D_XLP_MAIN; // A^(-1)
 
  double IN_PLANE_GAMMA;
  double OUT_OF_PLANE_GAMMA;
 
  double R2I; // A
  double R2F; // A
 
  double * twobodyfit;
};

class mbnrg_param_X_H2O_H2O {

public:
  mbnrg_param_X_H2O_H2O() {
    threebodyfit = new double[1831];
  };

  ~mbnrg_param_X_H2O_H2O() {
    delete[] threebodyfit;
  };

  void set_parameters(const std::string ion);

  double R3I; // A
  double R3F; // A
 
  double KHH_INTRA; // A^(-1)
  double KOH_INTRA; // A^(-1)
 
  double KHH; // A^(-1)
  double KOH; // A^(-1)
  double KOO; // A^(-1)
 
  double KXO; // A^(-1)
  double KXH; // A^(-1)
 
  double DHH_INTRA; // A
  double DOH_INTRA; // A
 
  double DHH; // A
  double DOH; // A
  double DOO; // A
 
  double DXO; // A
  double DXH; // A
 
  double * threebodyfit;

};


class mbnrg_param_ion_ion {

public:
  mbnrg_param_ion_ion() {
    deg = -1;
  };

  ~mbnrg_param_ion_ion() {
    if (deg != -1) {
      delete[] ki;
      delete[] ci;
    }
  };

  int deg;
  double deltaCA;
  double C6CA;
  double Ri;
  double Ro;
  double * ki;
  double * ci;

  void set_parameters(const std::string ion1, const std::string ion2);

};
#endif // PARAMS_H
