#ifndef POT_WATER_ION_H
#define POT_WATER_ION_H

#include "ttm4-es.h"
#include "watI-dispersion.h"
#include "watI-buckingham.h"


namespace x2o {

double link2f90_calc_pot(char*  coord_file, std::string atom_name);
struct pot_water_ion {

    void define_ion_params(const double*);

    // Energy for water and one ion
    double operator()(size_t nw, size_t nI, const double*) const;

    // Energy + Gradient for water and one ion
    double operator()(size_t nw, size_t nI, const double*, double*) const;

    // Energy for water
    double get_E_water(size_t nw, const double* pos);
    double get_E_water_1b(size_t nw, const double* pos);

    double get_E_water(size_t nw, const double* pos, double* );

    // Energy for water
    double get_E_water_ion(size_t nw, size_t nI, const double* pos);

    double get_E_water_ion(size_t nw, size_t nI, const double* pos, double * grd);

    // Energy for water
    double get_E_ion_ion(size_t nI1, size_t nI2, const double* pos);

    double get_E_ion_ion(size_t nI1, size_t nI2, const double* pos, double * grd);

    // Ion Ion mb-nrg
    double get_E_ion_ion_mbnrg(size_t nI1, size_t nI2, 
                                              const double* pos, double * grd);

    //Calculates Total Dipole
    void dip(const size_t&, const size_t&, double*);

    //Calculates 2B dipole
    void dipind(const size_t&, const size_t&, double*);

protected:
    mutable watI_dispersion m_watI_dispersion;
    mutable watI_buckingham m_watI_buckingham;
    mutable ttm::ttm4_es m_ttm4;
};

} // namespace x2o

#endif // POT_WATER_ION_H
