#include <cmath>

#include "watI-buckingham.h"

#include "orient.h"

namespace {

double buck(const double   a, const double   b,
	    const double* p1, const double* p2)
{
    const double r = tools::distance(p1, p2);
    return a*exp(-b*r);
}

double buck(const double   a, const double   b,
	    const double* p1, const double* p2, double* g1, double* g2)
{
    const double r = tools::distance(p1, p2);

    const double fac = a*exp(-b*r);
    //const double fac = a/b*exp(-b*r);

    for(size_t i = 0; i < 3; ++i){
	g1[i] -= (p1[i] - p2[i])*b/r*fac;
	g2[i] += (p1[i] - p2[i])*b/r*fac;
    }

    return fac;
}

}

namespace x2o {

double A_OX = 0;
double A_HX = 0;
double B_OX = 0;
double B_HX = 0;

//Set parameters
void  watI_buckingham::set_b_params(const double a_OX, const double b_OX,
                                    const double a_HX, const double b_HX)
{

    A_OX = a_OX;
    A_HX = a_HX;
    B_OX = b_OX;
    B_HX = b_HX;

}

// calculate distances 
double watI_buckingham::operator()(const double* wat, const double* ion)
{

    const double OX = buck(A_OX, B_OX, wat + 0, ion);
    const double HX = buck(A_HX, B_HX, wat + 3, ion)
                    + buck(A_HX, B_HX, wat + 6, ion);
    
    return OX + HX;

}

double watI_buckingham::operator()(const double* wat, const double* ion,
				       double* g_wat,     double* g_ion)
{

    const double OX = buck(A_OX, B_OX, wat + 0, ion, g_wat + 0, g_ion);
    const double HX = buck(A_HX, B_HX, wat + 3, ion, g_wat + 3, g_ion)
                    + buck(A_HX, B_HX, wat + 6, ion, g_wat + 6, g_ion);
    
    return OX + HX;

}

// calculate repulsive energies
double watI_buckingham::buckingham(const double rOX, const double rH1X, 
                                   const double rH2X,
                                   const double aOX, const double bOX,
                                   const double aHX, const double bHX)
{

    double Vbuckingham =  aOX*exp(-bOX*rOX)
						 +aHX*exp(-bHX*rH1X)
						 +aHX*exp(-bHX*rH2X);

    return Vbuckingham;
}

} // namespace x2o
