#include <cmath>

#include "tang-toennies.h"
#include "watI-dispersion.h"

#include "orient.h"

namespace x2o {

// Initialize parameters
double C6_OX = 0;
double C6_HX = 0;
double delta_OX = 0;
double delta_HX = 0;

//Set parameters
void  watI_dispersion::set_tt_params(const double c_OX, const double d_OX,
                                     const double c_HX, const double d_HX)
{

    C6_OX = c_OX;
    C6_HX = c_HX;
    delta_OX = d_OX;
    delta_HX = d_HX;

}

double watI_dispersion::operator()(const double* wat, const double* ion)
{

    const double OX6 = x6(C6_OX, delta_OX, wat + 0, ion);
    const double HX6 = x6(C6_HX, delta_HX, wat + 3, ion)
                     + x6(C6_HX, delta_HX, wat + 6, ion);

    return OX6 + HX6;

}

double watI_dispersion::operator()(const double* wat, const double* ion, 
				   double* g_wat, double* g_ion)
{

    const double OX6 = x6(C6_OX, delta_OX, wat + 0, ion, g_wat + 0, g_ion);
    const double HX6 = x6(C6_HX, delta_HX, wat + 3, ion, g_wat + 3, g_ion)
                     + x6(C6_HX, delta_HX, wat + 6, ion, g_wat + 6, g_ion);

    return OX6 + HX6;

}

// TODO: Since adding the MB-pol code, this fxn now duplicates what 
//       x6 does in tang-toennies.cpp. We should modify the fit code
//       to pass the coordinates instead of the distances to dispersion below
//       then use x6 instead
//
// calculate dispersion energies
double watI_dispersion::dispersion(const double rOX, const double rH1X, 
                                   const double rH2X,
                                   const double dOX, const double dHX,
                                   const double cOX, const double cHX)
{
    double InvROX, InvRH1X, InvRH2X;
    double ttOX, ttH1X, ttH2X;

    InvROX = 1/rOX;
    InvRH1X = 1/rH1X;
    InvRH2X = 1/rH2X;

    ttOX  = x2o::tang_toennies(6,dOX*rOX);
    ttH1X = x2o::tang_toennies(6,dHX*rH1X);
    ttH2X = x2o::tang_toennies(6,dHX*rH2X);

    double Vdispersion = -1.0*ttOX*cOX*pow(InvROX,6)
						 -1.0*ttH1X*cHX*pow(InvRH1X,6)
						 -1.0*ttH2X*cHX*pow(InvRH2X,6);

    return Vdispersion;
}

double mbnrg_dispersion(const double* w1, const double*  x,
                        const double C6_OX, const double C6_HX,
                        const double d6_OX, const double d6_HX,
                        double* g1, double* g2){

  const double* O  = w1;
  const double* H1 = w1 + 3;
  const double* H2 = w1 + 6;

  const double* X  =  x;

  double* gO  = g1;
  double* gH1 = g1 + 3;
  double* gH2 = g1 + 6;

  double* gX   = g2;

  const double  XH6 = x6(C6_HX , d6_HX , X , H1, gX , gH1)
                    + x6(C6_HX , d6_HX , X , H2, gX , gH2);

  const double  XO6 = x6(C6_OX , d6_OX , X , O, gX , gO);

  return  XH6 +  XO6;
}

} // namespace x2o
