#ifdef HAVE_CONFIG_H
#   include "config.h"
#endif // HAVE_CONFIG_H

#include <cmath>
#include <cassert>

#include "x3b-h2o-ion-v2x.h"
////////////////////////////////////////////////////////////////////////////////

namespace {

//----------------------------------------------------------------------------//

double var(const double& k,
           const double& r0,
           const double* a1, const double* a2)
{
    const double dx[3] = {a1[0] - a2[0],
                          a1[1] - a2[1],
                          a1[2] - a2[2]};

    const double dsq = dx[0]*dx[0] + dx[1]*dx[1] + dx[2]*dx[2];
    const double d = std::sqrt(dsq);

    return std::exp(-k*(d - r0));
}

//----------------------------------------------------------------------------//

void g_var(const double& g,
           const double& k,
           const double& r0,
           const double* a1, const double* a2,
                 double* g1,       double* g2)
{
    const double dx[3] = {a1[0] - a2[0],
                          a1[1] - a2[1],
                          a1[2] - a2[2]};

    const double dsq = dx[0]*dx[0] + dx[1]*dx[1] + dx[2]*dx[2];
    const double d = std::sqrt(dsq);

    const double gg = - k*g*std::exp(-k*(d - r0))/d;

    for (int i = 0; i < 3; ++i) {
        g1[i] += gg*dx[i];
        g2[i] -= gg*dx[i];
    }
}

//----------------------------------------------------------------------------//

} // namespace

////////////////////////////////////////////////////////////////////////////////

namespace h2o_ion {

//----------------------------------------------------------------------------//

//fit-ccsdtf12-avtz-cp-0.0-6.5-4th-iteration+troubling-configurations-alpha0.0001-w-constraints final fit 
void x3b_h2o_ion_v2x::set_param(mbnrg_param_X_H2O_H2O * param){
  r3i =         param->R3I; // A
  r3f =         param->R3F; // A
  
  kHH_intra =   param->KHH_INTRA; // A^(-1)
  kOH_intra =   param->KOH_INTRA; // A^(-1)
  
  kHH =         param->KHH; // A^(-1)
  kOH =         param->KOH; // A^(-1)
  kOO =         param->KOO; // A^(-1)
  
  kXO =        param->KXO; // A^(-1)
  kXH =        param->KXH; // A^(-1)
  
  dHH_intra =   param->DHH_INTRA; // A
  dOH_intra =   param->DOH_INTRA; // A
  
  dHH =         param->DHH; // A
  dOH =         param->DOH; // A
  dOO =         param->DOO; // A
  
  dXO =        param->DXO; // A
  dXH =        param->DXH; // A

  for (int i = 0; i< 1831; i++) {
    threebodyfit[i] = param->threebodyfit[i];
  }
}

//----------------------------------------------------------------------------//

double x3b_h2o_ion_v2x::f_switch(const double& r, double& g)
{
    if (r > r3f) {
        g = 0.0;
        return 0.0;
    } else if (r > r3i) {
        const double t1 = M_PI/(r3f - r3i);
        const double x = (r - r3i)*t1;
        g = - std::sin(x)*t1/2.0;
        return (1.0 + std::cos(x))/2.0;
    } else {
        g = 0.0;
        return 1.0;
    }
}

//----------------------------------------------------------------------------//

//double x3b_h2o_ion_v2x::eval(const double* w1,
//                     const double* w2,
//                     const double* x )
//{
//    const double* Oa  = w1;
//    const double* Ha1 = w1 + 3;
//    const double* Ha2 = w1 + 6;
//
//    const double* Ob  = w2;
//    const double* Hb1 = w2 + 3;
//    const double* Hb2 = w2 + 6;
//
//    const double* X   = x ;
//
//    double v[21];
//
//    v[0] = var(kHH_intra, dHH_intra, Ha1, Ha2);
//    v[1] = var(kHH_intra, dHH_intra, Hb1, Hb2);
//    v[2] = var(kOH_intra, dOH_intra,  Oa, Ha1);
//    v[3] = var(kOH_intra, dOH_intra,  Oa, Ha2);
//    v[4] = var(kOH_intra, dOH_intra,  Ob, Hb1);
//    v[5] = var(kOH_intra, dOH_intra,  Ob, Hb2);
//
//    v[6] = var(kHH, dHH, Ha1, Hb1);
//    v[7] = var(kHH, dHH, Ha1, Hb2);
//    v[8] = var(kHH, dHH, Ha2, Hb1);
//    v[9] = var(kHH, dHH, Ha2, Hb2);
//    v[10] = var(kOH, dOH,  Oa, Hb1);
//    v[11] = var(kOH, dOH,  Oa, Hb2);
//    v[12] = var(kOH, dOH,  Ob, Ha1);
//    v[13] = var(kOH, dOH,  Ob, Ha2);
//    v[14] = var(kOO, dOO,  Oa,  Ob);
//    v[15] = var(kXH, dXH,  X,  Ha1);
//    v[16] = var(kXH, dXH,  X,  Ha2);
//    v[17] = var(kXH, dXH,  X,  Hb1);
//    v[18] = var(kXH, dXH,  X,  Hb2);
//    v[19] = var(kXO, dXO,  Oa,  X);
//    v[20] = var(kXO, dXO,  Ob,  X);
//
//    double retval = poly_3b_h2o_ion_v1x::eval(threebodyfit, v);
//
//    double rab[3], rac[3], rbc[3];
//    double drab(0), drac(0), drbc(0);
//
//    for (int n = 0; n < 3; ++n) {
//        rab[n] = Oa[n] - Ob[n];
//        drab += rab[n]*rab[n];
//
//        rac[n] = Oa[n] -  X[n];
//        drac += rac[n]*rac[n];
//
//        rbc[n] = Ob[n] -  X[n];
//        drbc += rbc[n]*rbc[n];
//    }
//
//    drab = std::sqrt(drab);
//    drac = std::sqrt(drac);
//    drbc = std::sqrt(drbc);
//
//    double gab, gac, gbc;
//
//    const double sab = f_switch(drab, gab);
//    const double sac = f_switch(drac, gac);
//    const double sbc = f_switch(drbc, gbc);
//
//    const double s = sab*sac + sab*sbc + sac*sbc;
//
//    return s*retval;
//}

//----------------------------------------------------------------------------//

double x3b_h2o_ion_v2x::eval(const double* w1,
                     const double* w2,
                     const double* x ,
                     double* g1,
                     double* g2,
                     double* g3)
{
    const double* Oa  = w1;
    const double* Ha1 = w1 + 3;
    const double* Ha2 = w1 + 6;

    const double* Ob  = w2;
    const double* Hb1 = w2 + 3;
    const double* Hb2 = w2 + 6;

    const double* X   = x ;

    double v[21];

    v[0] = var(kHH_intra, dHH_intra, Ha1, Ha2);
    v[1] = var(kHH_intra, dHH_intra, Hb1, Hb2);
    v[2] = var(kOH_intra, dOH_intra,  Oa, Ha1);
    v[3] = var(kOH_intra, dOH_intra,  Oa, Ha2);
    v[4] = var(kOH_intra, dOH_intra,  Ob, Hb1);
    v[5] = var(kOH_intra, dOH_intra,  Ob, Hb2);

    v[6] = var(kHH, dHH, Ha1, Hb1);
    v[7] = var(kHH, dHH, Ha1, Hb2);
    v[8] = var(kHH, dHH, Ha2, Hb1);
    v[9] = var(kHH, dHH, Ha2, Hb2);
    v[10] = var(kOH, dOH,  Oa, Hb1);
    v[11] = var(kOH, dOH,  Oa, Hb2);
    v[12] = var(kOH, dOH,  Ob, Ha1);
    v[13] = var(kOH, dOH,  Ob, Ha2);
    v[14] = var(kOO, dOO,  Oa,  Ob);
    v[15] = var(kXH, dXH,  X,  Ha1);
    v[16] = var(kXH, dXH,  X,  Ha2);
    v[17] = var(kXH, dXH,  X,  Hb1);
    v[18] = var(kXH, dXH,  X,  Hb2);
    v[19] = var(kXO, dXO,  X,  Oa);
    v[20] = var(kXO, dXO,  X,  Ob);

    double g[21];
    double retval = poly_3b_h2o_ion_v1x::eval(threebodyfit, v, g);

    double rab[3], rac[3], rbc[3];
    double drab(0), drac(0), drbc(0);

    for (int n = 0; n < 3; ++n) {
        rab[n] = Oa[n] - Ob[n];
        drab += rab[n]*rab[n];

        rac[n] = Oa[n] -  X[n];
        drac += rac[n]*rac[n];

        rbc[n] = Ob[n] -  X[n];
        drbc += rbc[n]*rbc[n];
    }

    drab = std::sqrt(drab);
    drac = std::sqrt(drac);
    drbc = std::sqrt(drbc);

    double gab, gac, gbc;

    const double sab = f_switch(drab, gab);
    const double sac = f_switch(drac, gac);
    const double sbc = f_switch(drbc, gbc);

    const double s = sab*sac + sab*sbc + sac*sbc;

    for (int n = 0; n < 21; ++n)
        g[n] *= s;

    double* gOa  = g1;
    double* gHa1 = g1 + 3;
    double* gHa2 = g1 + 6;

    double* gOb  = g2;
    double* gHb1 = g2 + 3;
    double* gHb2 = g2 + 6;

    double* gX   = g3;

    g_var(g[0], kHH_intra, dHH_intra, Ha1, Ha2, gHa1, gHa2);
    g_var(g[1], kHH_intra, dHH_intra, Hb1, Hb2, gHb1, gHb2);
    g_var(g[2], kOH_intra, dOH_intra,  Oa, Ha1,  gOa, gHa1);
    g_var(g[3], kOH_intra, dOH_intra,  Oa, Ha2,  gOa, gHa2);
    g_var(g[4], kOH_intra, dOH_intra,  Ob, Hb1,  gOb, gHb1);
    g_var(g[5], kOH_intra, dOH_intra,  Ob, Hb2,  gOb, gHb2);

    g_var(g[6], kHH, dHH, Ha1, Hb1, gHa1, gHb1);
    g_var(g[7], kHH, dHH, Ha1, Hb2, gHa1, gHb2);
    g_var(g[8], kHH, dHH, Ha2, Hb1, gHa2, gHb1);
    g_var(g[9], kHH, dHH, Ha2, Hb2, gHa2, gHb2);
    g_var(g[10], kOH, dOH,  Oa, Hb1,  gOa, gHb1);
    g_var(g[11], kOH, dOH,  Oa, Hb2,  gOa, gHb2);
    g_var(g[12], kOH, dOH,  Ob, Ha1,  gOb, gHa1);
    g_var(g[13], kOH, dOH,  Ob, Ha2,  gOb, gHa2);
    g_var(g[14], kOO, dOO,  Oa,  Ob,  gOa,  gOb);
    g_var(g[15], kXH, dXH,  X,  Ha1,  gX,  gHa1);
    g_var(g[16], kXH, dXH,  X,  Ha2,  gX,  gHa2);
    g_var(g[17], kXH, dXH,  X,  Hb1,  gX,  gHb1);
    g_var(g[18], kXH, dXH,  X,  Hb2,  gX,  gHb2);
    g_var(g[19], kXO, dXO,  X,  Oa,   gX,  gOa);
    g_var(g[20], kXO, dXO,  X,  Ob,   gX,  gOb);

    // gradients of the switching function

    gab *= (sac + sbc)*retval/drab;
    gac *= (sab + sbc)*retval/drac;
    gbc *= (sab + sac)*retval/drbc;

    retval *= s;

    for (int n = 0; n < 3; ++n) {
        gOa[n] += gab*rab[n] + gac*rac[n];
        gOb[n] += gbc*rbc[n] - gab*rab[n];
         gX[n] -= gac*rac[n] + gbc*rbc[n];
    }

    return retval;
}

//----------------------------------------------------------------------------//

} // namespace h2o_ion

////////////////////////////////////////////////////////////////////////////////
