#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif // HAVE_CONFIG_H

#include <cmath>
#include <cassert>
#include <cstdlib>

#include <fstream>
#include <iomanip>
#include <iostream>
#include <algorithm>
#include <stdexcept>

#include "io-xyz.h"
#include "io-utils.h"

#include "ittm-Nion-water.h"

#include "xmin.h"

////////////////////////////////////////////////////////////////////////////////

namespace {

//----------------------------------------------------------------------------//
// cmdline arguments |
//-------------------+

static double grms_tol = 1.0e-6;
static int    maxit = 1000;

//static const char* output_file = 0;

void show_usage();
void parse_cmdline(int, char**);

std::vector<std::string> elements;
//----------------------------------------------------------------------------//

double grad_func(double* crd, double* grd, int *natm)
{
    return nion::get_E(crd, elements, elements.size() , grd);
}

//----------------------------------------------------------------------------//

} // namespace

////////////////////////////////////////////////////////////////////////////////

int main(int argc, char** argv)
{
    std::vector<double> crd;

    if (argc < 3) {
      std::cerr << "usage: test-pot-water-ion input.xyz output.xyz [n_it] [tol]"<< std::endl;
      return EXIT_FAILURE;
    }
    try {
        assert(argv[1]);

        std::ifstream ifs(argv[1]);

        if (!ifs)
            throw std::runtime_error("could not open the XYZ file");

        std::string comment;
        kit::io::load_xyz(ifs, comment, elements, crd);
    } catch (const std::exception& e) {
        std::cerr << " ** Error ** : " << e.what() << std::endl;
        return 1;
    }

    if (argc == 4) {
      maxit = atoi(argv[3]);
    } else if (argc == 5) {
      maxit = atoi(argv[3]);
      grms_tol = atof(argv[4]);
    }

    int natm = elements.size();

    double* xyz = new double[3*natm];
    double* grd = new double[3*natm];
//    double* xyz = new double[2*3*natm];
//    double* grd = xyz + 3*natm;

    // put first atom at origin
    std::copy(crd.begin(), crd.end(), xyz);
    for(int i = 0; i < natm; ++i)
      for(int j = 0; j < 3; ++j)
	       xyz[3*i + j] -= crd[j];

    kit::xmin_opt xo;
    kit::xmin_opt_init(&xo);

    xo.maxiter = maxit;
    xo.grms_tol = grms_tol;
    xo.print_level = 1;

    xo.method = 3; // 1 - PRCG, 2 -- LBGFS, 3 -- TNCG

    double energy, grms;

    energy = grad_func(xyz, grd, 0);
    const double energy0 = energy;

    kit::xmin(grad_func, &natm, xyz, grd, &energy, &grms, &xo);

    double grd2(0); // to double-check
    for (int n = 0; n < natm; ++n)
        grd2 += grd[n]*grd[n];

    const double energy1 = grad_func(xyz, grd, 0);
    std::cout << "<+> E = " << energy0 << " -> " << energy1
              << ", |g| = " << std::sqrt(grd2)
              << std::endl;

    std::stringstream ss("");
    ss << energy1;

    if (argv[2]) {
        std::cout << "<+> saving the XYZ to '"
                  << argv[2] << "'" << std::endl;
        std::ofstream ofs(argv[2]);

        std::copy(xyz, xyz + 3*natm, crd.begin());
        kit::io::save_xyz(ofs, ss.str(), elements, crd);
    }

    delete[] xyz;
    delete[] grd;

    return EXIT_SUCCESS;
}

////////////////////////////////////////////////////////////////////////////////
