#include "ittm-Nion-water.h"
#include <cmath>
//#define VERBOSE
//#define GRADIENTS

namespace {

static std::vector<tools::xyz_frame> molec;

} // namespace


int main(int argc, char** argv)
{
  
  // Check if we have the configs as an argument
  if (argc != 2) {
    std::cerr << "usage: test-pot-water-ion config.xyz " << std::endl; 
    return EXIT_FAILURE;
  }

  // Trying to read the file
  try {
    // Checking each one of the input files in the command line
    for(int i = 1; i < argc; ++i){
      tools::read_xyz(argv[i], molec);
    }
  } catch (const std::exception& e) {
    std::cerr << e.what() << std::endl;
  }

  double E_nograd(0), E_grad(0);
  double * grd;

  for (size_t i = 0 ; i< molec.size(); i++) {
    E_nograd = nion::get_E(molec[i].xyz,molec[i].at_name,molec[i].natm);
    std::cout << "Frame " << i << ": E_nograd= " << E_nograd << std::endl;

    grd = new double[3*molec[i].natm];
    E_grad = nion::get_E(molec[i].xyz,molec[i].at_name,molec[i].natm,grd);
    std::cout << "Frame " << i << ": E_grad= " << E_grad << std::endl;
    
#ifdef GRADIENTS
    const double eps = 1.0e-6;
    for(size_t n = 0; n < molec[i].natm * 3; ++n){
      const double x_orig = molec[i].xyz[n];

      molec[i].xyz[n] = x_orig + eps;
      const double Ep = nion::get_E(molec[i].xyz,molec[i].at_name,molec[i].natm,grd);
    
      molec[i].xyz[n] = x_orig + 2*eps;
      const double E2p = nion::get_E(molec[i].xyz,molec[i].at_name,molec[i].natm,grd);
    
      molec[i].xyz[n] = x_orig - 2*eps;
      const double E2m =  nion::get_E(molec[i].xyz,molec[i].at_name,molec[i].natm,grd);
    
      molec[i].xyz[n] = x_orig - eps;
      const double Em =  nion::get_E(molec[i].xyz,molec[i].at_name,molec[i].natm,grd);
    
      const double gfd = (8*(Ep - Em) - (E2p - E2m))/(12*eps);
      molec[i].xyz[n] = x_orig;
    
      std::cout << molec[i].at_name[n/3] << "   "  << "Analit: " << grd[n] << " Numerical: " << gfd 
                     << " Diff: " << std::fabs(grd[n] - gfd) << '\n';
    }
#endif

    delete[] grd;
  }


  return 0;
  
}


